﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Collections.Generic;
using System.Windows.Shapes;
using Infragistics.Controls.Charts;
using System.Linq;
using XamWebDataChartSLApp.Extensions;

namespace XamWebDataChartSLApp.Behaviors
{
    public partial class ChartBehaviors : DependencyObject
    {
        #region Crosshair behavior
        public static readonly DependencyProperty CrosshairHidingProperty =
            DependencyProperty.RegisterAttached("CrosshairHiding",
            typeof(CrosshairHidingBehavior), typeof(ChartBehaviors),
            new PropertyMetadata(null,
                (o, e) => CrosshairHidingChanged(
                    o as XamDataChart,
                    e.OldValue as CrosshairHidingBehavior,
                    e.NewValue as CrosshairHidingBehavior)));

        public static CrosshairHidingBehavior GetCrosshairHiding(DependencyObject target)
        {
            return target.GetValue(CrosshairHidingProperty) as CrosshairHidingBehavior;
        }

        public static void SetCrosshairHiding(DependencyObject target, CrosshairHidingBehavior behavior)
        {
            target.SetValue(CrosshairHidingProperty, behavior);
        }

        private static void CrosshairHidingChanged(
            XamDataChart chart,
            CrosshairHidingBehavior oldValue,
            CrosshairHidingBehavior newValue)
        {
            if (chart == null)
            {
                return;
            }
            if (oldValue != null)
            {
                oldValue.OnDetach(chart);
            }
            if (newValue != null)
            {
                newValue.OnAttach(chart);
            }
        }
        #endregion
    }
    public class CrosshairHidingBehavior
    {
        private Style _invisibleStyle;
        public CrosshairHidingBehavior()
        {
            _invisibleStyle = new Style(typeof(Line));
            _invisibleStyle.Setters.Add(
                new Setter(Shape.StrokeThicknessProperty, 0));
        }

        public bool HideHorizontalCrossHair { get; set; }
        public bool HideVerticalCrossHair { get; set; }

        public void OnAttach(XamDataChart chart)
        {
            chart.MouseEnter += Chart_MouseEnter;
        }

        public void OnDetach(XamDataChart chart)
        {
            chart.MouseEnter -= Chart_MouseEnter;
            MakeVisible(chart, _vertical);
            MakeVisible(chart, _horizontal);
            _vertical = null;
            _horizontal = null;
            _first = true;
        }

        private bool _first = true;
        private Line _vertical;
        private Line _horizontal;

        private void MakeInvisible(
            XamDataChart chart, Line crosshairLine)
        {
            crosshairLine.Style = _invisibleStyle;
        }
        private void MakeVisible(
            XamDataChart chart, Line crosshairLine)
        {
            if (chart == null || crosshairLine == null)
            {
                return;
            }
            crosshairLine.Style = chart.CrosshairLineStyle;
        }

        void Chart_MouseEnter(object sender, MouseEventArgs e)
        {
            XamDataChart chart = sender as XamDataChart;
            if (_first && chart != null)
            {
                var crosshairs = from line in chart.VisualDescendants()
                                 where line is Line &&
                                 (line as Line).Style == chart.CrosshairLineStyle
                                 select line as Line;
                List<Line> lines = crosshairs.ToList();

                // sometimes all values are 0. In that case try it again until we 
                // have one of the lines with different x values
                if (lines[0].X1 == lines[0].X2 && lines[1].X1 == lines[1].X2)
                    return;

                if (lines[0].X1 == lines[0].X2)
                {
                    _vertical = lines[0];
                    _horizontal = lines[1];
                }
                else
                {
                    _vertical = lines[1];
                    _horizontal = lines[0];
                }

                if (HideHorizontalCrossHair)
                {
                    MakeInvisible(chart, _horizontal);
                }
                if (HideVerticalCrossHair)
                {
                    MakeInvisible(chart, _vertical);
                }
                _first = false;
            }
        }
    }
}
