﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Infragistics.Controls.Charts;
using System.Windows.Data;
using System.Windows.Controls.Primitives;
using XamWebDataChartSLApp.Model;

namespace XamWebDataChartSLApp.Behaviors
{
    public partial class ChartBehaviors : DependencyObject
    {
        #region FloatingToolTip Behavior
        public static readonly DependencyProperty FloatingTooltipProperty =
           DependencyProperty.RegisterAttached("FloatingTooltip",
           typeof(FloatingTooltipBehavior), typeof(ChartBehaviors),
           new PropertyMetadata(null,
               (o, e) => FloatingTooltipChanged(
                   o as XamDataChart,
                   e.OldValue as FloatingTooltipBehavior,
                   e.NewValue as FloatingTooltipBehavior)));

        public static FloatingTooltipBehavior GetFloatingTooltip(DependencyObject target)
        {
            return target.GetValue(FloatingTooltipProperty)
                as FloatingTooltipBehavior;
        }

        public static void SetFloatingTooltip(DependencyObject target, FloatingTooltipBehavior behavior)
        {
            target.SetValue(FloatingTooltipProperty, behavior);
        }

        private static void FloatingTooltipChanged(
            XamDataChart chart,
            FloatingTooltipBehavior oldValue,
            FloatingTooltipBehavior newValue)
        {
            if (chart == null)
            {
                return;
            }
            if (oldValue != null)
            {
                oldValue.OnDetach(chart);
            }
            if (newValue != null)
            {
                newValue.OnAttach(chart);
            }
        }
        #endregion
    }

    public class FloatingTooltipBehavior : FrameworkElement
    {
        public static readonly DependencyProperty HoveredItemsProperty =
            DependencyProperty.Register("HoveredItems",
            typeof(SeriesItemInfoCollection),
            typeof(FloatingTooltipBehavior),
            new PropertyMetadata(null));

        public SeriesItemInfoCollection HoveredItems
        {
            get { return (SeriesItemInfoCollection)GetValue(HoveredItemsProperty); }
            set { SetValue(HoveredItemsProperty, value); }
        }

        private bool _isOverChart = false;
        private Popup _popup = new Popup();
        private ContentControl _content = new ContentControl();
        private Panel _container;
        private XamDataChart _owner = null;

        private DataTemplate _tooltipTemplate;

        public DataTemplate TooltipTemplate
        {
            get { return _tooltipTemplate; }
            set
            {
                _tooltipTemplate = value;
                _content.ContentTemplate = _tooltipTemplate;
            }
        }

        protected bool IsOverChart
        {
            get { return _isOverChart; }
            set
            {
                bool last = _isOverChart;
                _isOverChart = value;
                if (_isOverChart && !last)
                {
                    ShowPopup();
                }
                if (!_isOverChart && last)
                {
                    HidePopup();
                }
            }
        }

        private void HidePopup()
        {
            _popup.IsOpen = false;
        }

        private void ShowPopup()
        {
            _popup.IsOpen = true;
        }

        public void OnAttach(XamDataChart chart)
        {
            if (_owner != null)
            {
                OnDetach(_owner);
            }
            _owner = chart;
            this.DataContext = chart;
            chart.MouseLeave += Chart_MouseLeave;
            chart.MouseMove += Chart_MouseMove;

            _popup.IsOpen = false;
            _popup.Child = _content;
            _content.ContentTemplate = TooltipTemplate;
            if (_content.Content == null)
            {
                _content.SetBinding(
                    ContentControl.ContentProperty,
                    new Binding("HoveredItems")
                    {
                        Source = this
                    });
            }

            if (chart.Parent != null && chart.Parent is Panel)
            {
                _container = chart.Parent as Panel;
                _container.Children.Add(_popup);
            }
        }

        public void OnDetach(XamDataChart chart)
        {
            if (_owner != chart)
            {
                return;
            }
            chart.MouseLeave -= Chart_MouseLeave;
            chart.MouseMove -= Chart_MouseMove;
            IsOverChart = false;
            if (_container != null)
            {
                _container.Children.Remove(_popup);
            }
            _owner = null;
        }

        void Chart_MouseMove(object sender, MouseEventArgs e)
        {
            // only show the popup if there is data to display
            if (HoveredItems != null && HoveredItems.Count > 0)
            {
                // if you pass the chart control in, the point is relative to the chart
                // rather than the root control
                //SetPopupOffsets(e.GetPosition(_owner));
                SetPopupOffsets(e.GetPosition(null));
                IsOverChart = true;
            }
        }

        private void SetPopupOffsets(Point point)
        {
            _popup.VerticalOffset = point.Y;

            // need to move the popup from the right side of the pointer
            // to the left side so it does not slide out of view. Problem
            // is that we do not know the size of the popup at this time.
            if (point.X + _popup.Child.DesiredSize.Width >= _owner.ActualWidth)
            {
                _popup.HorizontalOffset = point.X - _popup.Child.DesiredSize.Width - 10;
            }
            else
                _popup.HorizontalOffset = point.X + 10;
        }

        void Chart_MouseLeave(object sender, MouseEventArgs e)
        {
            IsOverChart = false;
            if (HoveredItems != null) { HoveredItems.Clear(); }
        }
    }
}
